/**
 * @package     Opentherm library for ESP-IDF framework - EXAMPLE
 * @author:     Mikhail Sazanof
 * @copyright   Copyright (C) 2024 - Sazanof.ru
 * @licence     MIT
 */

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "opentherm.h"
#include <esp_err.h>
#include <esp_log.h>
#include <stdio.h>

#define GPIO_OT_IN CONFIG_OT_IN_PIN
#define GPIO_OT_OUT CONFIG_OT_OUT_PIN
#define ESP_INTR_FLAG_DEFAULT 0

volatile float g_dhwTemp = 0;
volatile float g_chTemp = 0;
volatile bool g_fault = false;
static int g_targetDHWTemp = 59;
static int g_targetCHTemp = 60;

static const char *g_T = "OT";

static void IRAM_ATTR esp_ot_process_response_callback(unsigned long response, open_therm_response_status_t esp_ot_response_status)
{
    // ESP_DRAM_LOGW(T, "Response from esp_ot_process_responseCallback!");
    // ESP_DRAM_LOGW(T, "var response From CB: %lu", response);
    // ESP_DRAM_LOGW(T, "var esp_ot_response_status from CB: %d", (int)esp_ot_response_status);
}

void esp_ot_control_task_handler(void *pvParameter)
{
    while (1) {
        unsigned long status = esp_ot_set_boiler_status(false, false, false, false, false);

        ESP_LOGI(g_T, "====== OPENTHERM =====");
        ESP_LOGI(g_T, "Free heap size before: %ld", esp_get_free_heap_size());
        open_therm_response_status_t esp_ot_response_status = esp_ot_get_last_response_status();
        if (esp_ot_response_status == OT_STATUS_SUCCESS) {
            ESP_LOGI(g_T, "Central Heating: %s", esp_ot_is_central_heating_active(status) ? "ON" : "OFF");
            ESP_LOGI(g_T, "Hot Water: %s", esp_ot_is_hot_water_active(status) ? "ON" : "OFF");
            ESP_LOGI(g_T, "Flame: %s", esp_ot_is_flame_on(status) ? "ON" : "OFF");
            g_fault = esp_ot_is_fault(status);
            ESP_LOGI(g_T, "Fault: %s", g_fault ? "YES" : "NO");
            if (g_fault) {
                ot_reset();
            }
            esp_ot_set_boiler_temperature(g_targetCHTemp);
            ESP_LOGI(g_T, "Set CH Temp to: %i", g_targetCHTemp);

            esp_ot_set_dhw_setpoint(g_targetDHWTemp);
            ESP_LOGI(g_T, "Set DHW Temp to: %i", g_targetDHWTemp);

            g_dhwTemp = esp_ot_get_dhw_temperature();
            ESP_LOGI(g_T, "DHW Temp: %.1f", g_dhwTemp);

            g_chTemp = esp_ot_get_boiler_temperature();
            ESP_LOGI(g_T, "CH Temp: %.1f", g_chTemp);

            float pressure = esp_ot_get_pressure();
            ESP_LOGI(g_T, "Pressure: %.1f", pressure);

            unsigned long slaveProductVersion = esp_ot_get_slave_product_version();
            ESP_LOGI(g_T, "Slave Version: %08lX", slaveProductVersion);

            float slaveOTVersion = esp_ot_get_slave_ot_version();
            ESP_LOGI(g_T, "Slave OT Version: %.1f", slaveOTVersion);
        } else if (esp_ot_response_status == OT_STATUS_TIMEOUT) {
            ESP_LOGE(g_T, "OT Communication Timeout");
        } else if (esp_ot_response_status == OT_STATUS_INVALID) {
            ESP_LOGE(g_T, "OT Communication Invalid");
        } else if (esp_ot_response_status == OT_STATUS_NONE) {
            ESP_LOGE(g_T, "OpenTherm not initialized");
        }

        if (g_fault) {
            ESP_LOGE(g_T, "Fault Code: %i", esp_ot_get_fault());
        }
        ESP_LOGI(g_T, "Free heap size after: %ld", esp_get_free_heap_size());
        ESP_LOGI(g_T, "====== OPENTHERM =====\r\n\r\n");

        vTaskDelay(1000 / portTICK_PERIOD_MS);
    }
}

void app_main()
{
    esp_ot_init(GPIO_OT_IN, GPIO_OT_OUT, false, esp_ot_process_response_callback);

    xTaskCreate(esp_ot_control_task_handler, g_T, configMINIMAL_STACK_SIZE * 4, NULL, 3, NULL);
    vTaskSuspend(NULL);
}
